@@ -26,13 +26,8 @@ from typing import (
 from robot.errors import VariableError
 from robot.libraries import STDLIBS
 from robot.parsing.lexer.tokens import Token
-from robot.parsing.model.blocks import (
-    Keyword,
-    SettingSection,
-    TestCase,
-    VariableSection,
-)
-from robot.parsing.model.statements import Arguments, Statement
+from robot.parsing.model.blocks import Keyword, SettingSection, TestCase, VariableSection
+from robot.parsing.model.statements import Arguments, Fixture, Statement, Timeout
 from robot.parsing.model.statements import LibraryImport as RobotLibraryImport
 from robot.parsing.model.statements import ResourceImport as RobotResourceImport
 from robot.parsing.model.statements import (
@@ -99,6 +94,7 @@ from .library_doc import (
     KeywordError,
     KeywordMatcher,
     LibraryDoc,
+    resolve_robot_variables,
 )
 
 EXTRACT_COMMENT_PATTERN = re.compile(r".*(?:^ *|\t+| {2,})#(?P<comment>.*)$")
@@ -181,74 +177,23 @@ class VariablesVisitor(Visitor):
             )
 
 
-class BlockVariableVisitor(Visitor):
+class VariableVisitorBase(Visitor):
     def __init__(
         self,
-        library_doc: LibraryDoc,
-        global_variables: List[VariableDefinition],
-        source: str,
+        namespace: "Namespace",
+        nodes: Optional[List[ast.AST]] = None,
         position: Optional[Position] = None,
         in_args: bool = True,
     ) -> None:
         super().__init__()
-        self.library_doc = library_doc
-        self.global_variables = global_variables
-        self.source = source
+        self.namespace = namespace
+        self.nodes = nodes
         self.position = position
         self.in_args = in_args
 
         self._results: Dict[str, VariableDefinition] = {}
         self.current_kw_doc: Optional[KeywordDoc] = None
-        self._var_statements_vars: List[VariableDefinition] = []
-
-    def get(self, model: ast.AST) -> List[VariableDefinition]:
-        self._results = {}
-
-        self.visit(model)
-
-        return list(self._results.values())
-
-    def visit(self, node: ast.AST) -> None:
-        if self.position is None or self.position >= range_from_node(node).start:
-            super().visit(node)
-
-    def visit_Keyword(self, node: ast.AST) -> None:  # noqa: N802
-        try:
-            self.generic_visit(node)
-        finally:
-            self.current_kw_doc = None
-
-    def visit_KeywordName(self, node: Statement) -> None:  # noqa: N802
-        from .model_helper import ModelHelper
-
-        name_token = node.get_token(Token.KEYWORD_NAME)
-
-        if name_token is not None and name_token.value:
-            keyword = ModelHelper.get_keyword_definition_at_token(self.library_doc, name_token)
-            self.current_kw_doc = keyword
-
-            for variable_token in filter(
-                lambda e: e.type == Token.VARIABLE,
-                tokenize_variables(name_token, identifiers="$", ignore_errors=True),
-            ):
-                if variable_token.value:
-                    match = search_variable(variable_token.value, "$", ignore_errors=True)
-                    if match.base is None:
-                        continue
-                    name = match.base.split(":", 1)[0]
-                    full_name = f"{match.identifier}{{{name}}}"
-                    var_token = strip_variable_token(variable_token)
-                    var_token.value = name
-                    self._results[full_name] = ArgumentDefinition(
-                        name=full_name,
-                        name_token=var_token,
-                        line_no=variable_token.lineno,
-                        col_offset=variable_token.col_offset,
-                        end_line_no=variable_token.lineno,
-                        end_col_offset=variable_token.end_col_offset,
-                        source=self.source,
-                        keyword_doc=self.current_kw_doc,
-                    )
+        self.current_kw: Optional[Keyword] = None
 
     def get_variable_token(self, token: Token) -> Optional[Token]:
         return next(
@@ -263,6 +208,27 @@ class BlockVariableVisitor(Visitor):
             None,
         )
 
+
+class ArgumentVisitor(VariableVisitorBase):
+    def __init__(
+        self,
+        namespace: "Namespace",
+        nodes: Optional[List[ast.AST]],
+        position: Optional[Position],
+        in_args: bool,
+        current_kw_doc: Optional[KeywordDoc],
+    ) -> None:
+        super().__init__(namespace, nodes, position, in_args)
+
+        self.current_kw_doc: Optional[KeywordDoc] = current_kw_doc
+
+    def get(self, model: ast.AST) -> Dict[str, VariableDefinition]:
+        self._results = {}
+
+        self.visit(model)
+
+        return self._results
+
     def visit_Arguments(self, node: Statement) -> None:  # noqa: N802
         args: List[str] = []
 
@@ -290,7 +256,7 @@ class BlockVariableVisitor(Visitor):
                             col_offset=argument.col_offset,
                             end_line_no=argument.lineno,
                             end_col_offset=argument.end_col_offset,
-                            source=self.source,
+                            source=self.namespace.source,
                             keyword_doc=self.current_kw_doc,
                         )
                         self._results[argument.value] = arg_def
@@ -298,6 +264,69 @@ class BlockVariableVisitor(Visitor):
             except VariableError:
                 pass
 
+
+class OnlyArgumentsVisitor(VariableVisitorBase):
+    def get(self, model: ast.AST) -> List[VariableDefinition]:
+        self._results = {}
+
+        self.visit(model)
+
+        return list(self._results.values())
+
+    def visit(self, node: ast.AST) -> None:
+        if self.position is None or self.position >= range_from_node(node).start:
+            super().visit(node)
+
+    def visit_Keyword(self, node: ast.AST) -> None:  # noqa: N802
+        self.current_kw = cast(Keyword, node)
+        try:
+            self.generic_visit(node)
+        finally:
+            self.current_kw = None
+            self.current_kw_doc = None
+
+    def visit_KeywordName(self, node: Statement) -> None:  # noqa: N802
+        from .model_helper import ModelHelper
+
+        name_token = node.get_token(Token.KEYWORD_NAME)
+
+        if name_token is not None and name_token.value:
+            keyword = ModelHelper.get_keyword_definition_at_token(self.namespace.get_library_doc(), name_token)
+            self.current_kw_doc = keyword
+
+            for variable_token in filter(
+                lambda e: e.type == Token.VARIABLE,
+                tokenize_variables(name_token, identifiers="$", ignore_errors=True),
+            ):
+                if variable_token.value:
+                    match = search_variable(variable_token.value, "$", ignore_errors=True)
+                    if match.base is None:
+                        continue
+                    name = match.base.split(":", 1)[0]
+                    full_name = f"{match.identifier}{{{name}}}"
+                    var_token = strip_variable_token(variable_token)
+                    var_token.value = name
+                    self._results[full_name] = ArgumentDefinition(
+                        name=full_name,
+                        name_token=var_token,
+                        line_no=variable_token.lineno,
+                        col_offset=variable_token.col_offset,
+                        end_line_no=variable_token.lineno,
+                        end_col_offset=variable_token.end_col_offset,
+                        source=self.namespace.source,
+                        keyword_doc=self.current_kw_doc,
+                    )
+
+            if self.current_kw is not None:
+                args = ArgumentVisitor(
+                    self.namespace, self.nodes, self.position, self.in_args, self.current_kw_doc
+                ).get(self.current_kw)
+                if args:
+                    self._results.update(args)
+
+
+class BlockVariableVisitor(OnlyArgumentsVisitor):
+
     def visit_ExceptHeader(self, node: Statement) -> None:  # noqa: N802
         variables = node.get_tokens(Token.VARIABLE)[:1]
         if variables and is_scalar_assign(variables[0].value):
@@ -312,12 +341,48 @@ class BlockVariableVisitor(Visitor):
                         col_offset=variable.col_offset,
                         end_line_no=variable.lineno,
                         end_col_offset=variable.end_col_offset,
-                        source=self.source,
+                        source=self.namespace.source,
                     )
 
             except VariableError:
                 pass
 
+    def _get_var_name(self, original: str, position: Position, require_assign: bool = True) -> Optional[str]:
+        robot_variables = resolve_robot_variables(
+            str(self.namespace.imports_manager.root_folder),
+            str(Path(self.namespace.source).parent) if self.namespace.source else ".",
+            self.namespace.imports_manager.get_resolvable_command_line_variables(),
+            variables=self.namespace.get_resolvable_variables(),
+        )
+
+        try:
+            replaced = robot_variables.replace_string(original)
+        except VariableError:
+            replaced = original
+        try:
+            name = self._resolve_var_name(replaced, robot_variables)
+        except ValueError:
+            name = original
+        match = search_variable(name, identifiers="$@&")
+        match.resolve_base(robot_variables)
+        valid = match.is_assign() if require_assign else match.is_variable()
+        if not valid:
+            return None
+        return str(match)
+
+    def _resolve_var_name(self, name: str, variables: Any) -> str:
+        if name.startswith("\\"):
+            name = name[1:]
+        if len(name) < 2 or name[0] not in "$@&":
+            raise ValueError
+        if name[1] != "{":
+            name = f"{name[0]}{{{name[1:]}}}"
+        match = search_variable(name, identifiers="$@&", ignore_errors=True)
+        match.resolve_base(variables)
+        if not match.is_assign():
+            raise ValueError
+        return str(match)
+
     def visit_KeywordCall(self, node: Statement) -> None:  # noqa: N802
         # TODO  analyze "Set Local/Global/Suite Variable"
 
@@ -341,12 +406,65 @@ class BlockVariableVisitor(Visitor):
                             col_offset=variable_token.col_offset,
                             end_line_no=variable_token.lineno,
                             end_col_offset=variable_token.end_col_offset,
-                            source=self.source,
+                            source=self.namespace.source,
                         )
 
             except VariableError:
                 pass
 
+        keyword_token = node.get_token(Token.KEYWORD)
+        if keyword_token is None or not keyword_token.value:
+            return
+
+        keyword = self.namespace.find_keyword(keyword_token.value, raise_keyword_error=False)
+        if keyword is None:
+            return
+
+        if keyword.libtype == "LIBRARY" and keyword.libname == "BuiltIn":
+            var_type = None
+            if keyword.name == "Set Suite Variable":
+                var_type = VariableDefinition
+            elif keyword.name == "Set Global Variable":
+                var_type = GlobalVariableDefinition
+            elif keyword.name == "Set Test Variable" or keyword.name == "Set Task Variable":
+                var_type = TestVariableDefinition
+            elif keyword.name == "Set Local Variable":
+                var_type = LocalVariableDefinition
+            else:
+                return
+            try:
+                variable = node.get_token(Token.ARGUMENT)
+                if variable is None:
+                    return
+
+                position = range_from_node(node).start
+                position.character = 0
+                var_name = self._get_var_name(variable.value, position)
+
+                if var_name is None or not is_variable(var_name):
+                    return
+
+                var = var_type(
+                    name=var_name,
+                    name_token=strip_variable_token(variable),
+                    line_no=variable.lineno,
+                    col_offset=variable.col_offset,
+                    end_line_no=variable.lineno,
+                    end_col_offset=variable.end_col_offset,
+                    source=self.namespace.source,
+                )
+
+                if var_name not in self._results or type(self._results[var_name]) != type(var):
+                    if isinstance(var, LocalVariableDefinition) or not any(
+                        l for l in self.namespace.get_global_variables() if l.matcher == var.matcher
+                    ):
+                        self._results[var_name] = var
+                    else:
+                        self._results.pop(var_name, None)
+
+            except VariableError:
+                pass
+
     def visit_InlineIfHeader(self, node: Statement) -> None:  # noqa: N802
         for assign_token in node.get_tokens(Token.ASSIGN):
             variable_token = self.get_variable_token(assign_token)
@@ -368,7 +486,7 @@ class BlockVariableVisitor(Visitor):
                             col_offset=variable_token.col_offset,
                             end_line_no=variable_token.lineno,
                             end_col_offset=variable_token.end_col_offset,
-                            source=self.source,
+                            source=self.namespace.source,
                         )
 
             except VariableError:
@@ -386,7 +504,7 @@ class BlockVariableVisitor(Visitor):
                     col_offset=variable_token.col_offset,
                     end_line_no=variable_token.lineno,
                     end_col_offset=variable_token.end_col_offset,
-                    source=self.source,
+                    source=self.namespace.source,
                 )
 
     def visit_Var(self, node: Statement) -> None:  # noqa: N802
@@ -421,14 +539,12 @@ class BlockVariableVisitor(Visitor):
                 col_offset=variable.col_offset,
                 end_line_no=variable.lineno,
                 end_col_offset=variable.end_col_offset,
-                source=self.source,
+                source=self.namespace.source,
             )
 
-            self._var_statements_vars.append(var)
-
             if var_name not in self._results or type(self._results[var_name]) != type(var):
                 if isinstance(var, LocalVariableDefinition) or not any(
-                    l for l in self.global_variables if l.matcher == var.matcher
+                    l for l in self.namespace.get_global_variables() if l.matcher == var.matcher
                 ):
                     self._results[var_name] = var
                 else:
@@ -905,10 +1021,12 @@ class Namespace:
         nodes: Optional[List[ast.AST]] = None,
         position: Optional[Position] = None,
         skip_commandline_variables: bool = False,
+        skip_local_variables: bool = False,
     ) -> Iterator[Tuple[VariableMatcher, VariableDefinition]]:
         yielded: Dict[VariableMatcher, VariableDefinition] = {}
 
         test_or_keyword = None
+        test_or_keyword_nodes = None
 
         if nodes:
             test_or_keyword_nodes = list(
@@ -919,19 +1037,23 @@ class Namespace:
             )
             test_or_keyword = test_or_keyword_nodes[0] if test_or_keyword_nodes else None
 
+        in_args = isinstance(test_or_keyword_nodes[-1], Arguments) if test_or_keyword_nodes else False
+        only_args = (
+            isinstance(test_or_keyword_nodes[-1], (Arguments, Fixture, Timeout)) if test_or_keyword_nodes else False
+        )
+
         for var in chain(
             *[
                 (
                     (
-                        BlockVariableVisitor(
-                            self.get_library_doc(),
-                            self.get_global_variables(),
-                            self.source,
+                        (OnlyArgumentsVisitor if only_args else BlockVariableVisitor)(
+                            self,
+                            nodes,
                             position,
-                            isinstance(test_or_keyword_nodes[-1], Arguments) if nodes else False,
+                            in_args,
                         ).get(test_or_keyword)
                     )
-                    if test_or_keyword is not None
+                    if test_or_keyword is not None and not skip_local_variables
                     else []
                 )
             ],
@@ -997,6 +1119,7 @@ class Namespace:
         nodes: Optional[List[ast.AST]] = None,
         position: Optional[Position] = None,
         skip_commandline_variables: bool = False,
+        skip_local_variables: bool = False,
         ignore_error: bool = False,
     ) -> Optional[VariableDefinition]:
         self.ensure_initialized()
@@ -1021,6 +1144,7 @@ class Namespace:
                 nodes,
                 position,
                 skip_commandline_variables=skip_commandline_variables,
+                skip_local_variables=skip_local_variables,
             ):
                 if matcher == m:
                     return v
@@ -1039,6 +1163,7 @@ class Namespace:
         top_level: bool = False,
         source: Optional[str] = None,
         parent_import: Optional[Import] = None,
+        parent_source: Optional[str] = None,
     ) -> Optional[LibraryEntry]:
         result: Optional[LibraryEntry] = None
         try:
@@ -1220,6 +1345,26 @@ class Namespace:
                     source=DIAGNOSTICS_SOURCE_NAME,
                     code=type(e).__qualname__,
                 )
+            elif parent_import is not None:
+                self.append_diagnostics(
+                    range=parent_import.range,
+                    message="Import definition contains errors.",
+                    severity=DiagnosticSeverity.ERROR,
+                    source=DIAGNOSTICS_SOURCE_NAME,
+                    code=Error.IMPORT_CONTAINS_ERRORS,
+                    related_information=(
+                        (
+                            [
+                                DiagnosticRelatedInformation(
+                                    location=Location(str(Uri.from_path(parent_source)), value.range),
+                                    message=str(e),
+                                ),
+                            ]
+                        )
+                        if parent_source
+                        else None
+                    ),
+                )
         finally:
             self._reset_global_variables()
 
@@ -1234,6 +1379,7 @@ class Namespace:
         variables: Optional[Dict[str, Any]] = None,
         source: Optional[str] = None,
         parent_import: Optional[Import] = None,
+        parent_source: Optional[str] = None,
         depth: int = 0,
     ) -> Optional[Dict[str, Any]]:
 
@@ -1251,6 +1397,7 @@ class Namespace:
                     top_level=top_level,
                     source=source,
                     parent_import=parent_import,
+                    parent_source=parent_source if parent_source else source,
                 )
 
                 if entry is not None:
@@ -1274,6 +1421,7 @@ class Namespace:
                                     variables=variables,
                                     source=entry.library_doc.source,
                                     parent_import=imp if top_level else parent_import,
+                                    parent_source=parent_source if top_level else source,
                                     depth=depth + 1,
                                 )
                             except (SystemExit, KeyboardInterrupt):
